--  ls2.adb
--    Purpose: list contents of directory or directories
--    Action:  if no args, use .  else list files in args
--    BUG:     try './ls2 /tmp'

with Ada.Command_Line;
with Ada.Integer_Text_IO;
with Ada.Text_IO;
with Interfaces.C.Strings;
with POSIX.C;
with POSIX.Calendar;
with POSIX.File_Status;
with POSIX.Files;
with POSIX.Group_Database;
with POSIX.Permissions.Text_IO;
with POSIX.Process_Identification;
with POSIX.Strings;
with POSIX.User_Database;

procedure ls is
   use POSIX.File_Status;
   use POSIX.Files;
   use POSIX.Process_Identification;

   function Mode_To_Letters (Info : in     Status) return String is
      use POSIX.Permissions.Text_IO;
   begin
      if Is_Directory (File_Status => Info) then
         return 'd' & Image (Permission_Set_Of (Info));
      elsif Is_Character_Special_File (File_Status => Info) then
         return 'c' & Image (Permission_Set_Of (Info));
      elsif Is_Block_Special_File (File_Status => Info) then
         return 'b' & Image (Permission_Set_Of (Info));
      else
         return '-' & Image (Permission_Set_Of (Info));
      end if;
   end Mode_To_Letters;

   function Name_Of (ID : in     User_ID) return String is
      use POSIX.User_Database;
      use POSIX.Process_Identification;
      use POSIX.Strings;
      User : constant User_Database_Item := Get_User_Database_Item (ID);
   begin
      return Format (User_Name_Of (User), Width => 8);
   exception
      when others =>
         return Format (Image (ID), Width => 8);
   end Name_Of;

   function Name_Of (ID : in     Group_ID) return String is
      use POSIX.Group_Database;
      use POSIX.Process_Identification;
      use POSIX.Strings;
      Group : constant Group_Database_Item := Get_Group_Database_Item (ID);
   begin
      return Format (Group_Name_Of (Group), Width => 8);
   exception
      when others =>
         return Format (Image (ID), Width => 8);
   end Name_Of;

   procedure Put (Item : in     POSIX.Calendar.POSIX_Time) is
      use Ada.Text_IO;
      use Interfaces.C.Strings;
      use POSIX.Calendar;
      function ctime (timep : access POSIX.C.time_t) return chars_ptr;
      pragma Import (C, ctime);
      Buffer : aliased POSIX.C.time_t
        := POSIX.C.time_t (POSIX.Get_Seconds (To_Timespec (Item)));
   begin
      Put (Value (ctime (Buffer'Access)) (5 .. 16));
   end Put;

   procedure List_File (D_Entry : in     Directory_Entry;
                        Quit    : in out Boolean) is
      use Ada.Integer_Text_IO;
      use Ada.Text_IO;
      use POSIX.Process_Identification;
      use POSIX.Strings;
      Name : constant POSIX.POSIX_String := Filename_Of (D_Entry);
      Info : Status;
   begin
      Info := Get_File_Status (Name);

      Put (Mode_To_Letters (Info));
      Put (" ");
      Put (Link_Count_Of (Info), Width => 2);
      Put (" ");
      Put (Name_Of (Owner_Of (Info)));
      Put (" ");
      Put (Name_Of (Group_Of (Info)));
      Put (" ");
      if Is_Regular_File (Info) then
         Put (Natural (Size_Of (Info)), Width => 8);
      else
         Put ("       -");
      end if;
      Put (" ");
      Put (Last_Modification_Time_Of (Info));
      Put (" ");
      Put_Line (To_Ada_String (Name));
   exception
      when others =>
         Put_Line (File => Standard_Error,
                   Item => "Exception in procedure List_File (while " &
                           "showing """ & To_Ada_String (Name) & """).");
   end List_File;

   procedure List is new For_Every_Directory_Entry (Action => List_File);
begin
   if Ada.Command_Line.Argument_Count < 1 then
      List (".");
   else
      for Index in 1 .. Ada.Command_Line.Argument_Count loop
         declare
            use Ada.Text_IO;
            Directory : String renames Ada.Command_Line.Argument (Index);
         begin
            Put_Line (Directory & ":");
            List (POSIX.To_POSIX_String (Directory));
         exception
            when others =>
               Put_Line (File => Standard_Error,
                         Item => Ada.Command_Line.Command_Name &
                                 ": cannot open " & Directory);
         end;
      end loop;
   end if;
end ls;
